"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const option_1 = require("@reactive-js/option");
const parser_combinators_1 = require("@reactive-js/parser-combinators");
const pipe_1 = require("@reactive-js/pipe");
const httpGrammar_1 = require("./httpGrammar");
const httpHeaders_1 = require("./httpHeaders");
const mediaType_1 = require("./mediaType");
const parseTokenList = pipe_1.pipe(httpGrammar_1.pToken, httpGrammar_1.httpList, parser_combinators_1.parseWith);
exports.parseHttpContentFromHeaders = (headers, body) => {
    var _a, _b, _c;
    const contentEncodingString = (_a = httpHeaders_1.getHeaderValue(headers, "Content-Encoding")) !== null && _a !== void 0 ? _a : "";
    const contentEncodings = parseTokenList(contentEncodingString);
    const contentLengthHeader = (_b = httpHeaders_1.getHeaderValue(headers, "Content-Length")) !== null && _b !== void 0 ? _b : "-1";
    const contentLength = ~~contentLengthHeader;
    const contentType = mediaType_1.parseMediaType((_c = httpHeaders_1.getHeaderValue(headers, "Content-Type")) !== null && _c !== void 0 ? _c : "");
    return option_1.isNone(contentType)
        ? option_1.none
        : {
            body,
            contentEncodings,
            contentLength,
            contentType,
        };
};
exports.writeHttpContentHeaders = (content, writeHeader) => {
    const { contentLength, contentType, contentEncodings } = content;
    if (contentLength > 0) {
        writeHeader("Content-Length", contentLength.toString(10));
    }
    writeHeader("Content-Type", mediaType_1.mediaTypeToString(contentType));
    if (contentEncodings.length > 0) {
        writeHeader("Content-Encoding", contentEncodings.join(", "));
    }
};
//# sourceMappingURL=httpContent.js.map