"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const option_1 = require("@reactive-js/option");
const parser_combinators_1 = require("@reactive-js/parser-combinators");
const pipe_1 = require("@reactive-js/pipe");
const httpGrammar_1 = require("./httpGrammar");
const httpHeaders_1 = require("./httpHeaders");
const mediaType_1 = require("./mediaType");
const weightedParamComparator = (a, b) => {
    var _a, _b;
    const qA = ((_a = Number.parseFloat(a["q"])) !== null && _a !== void 0 ? _a : 1) * 1000;
    const qB = ((_b = Number.parseFloat(b["q"])) !== null && _b !== void 0 ? _b : 1) * 1000;
    return qA - qB;
};
const mediaRangeCompare = (a, b) => weightedParamComparator(a.params, b.params);
const mediaTypeToMediaRange = ({ type, subtype }) => ({
    type,
    subtype,
});
const parseAccept = pipe_1.pipe(mediaType_1.pMediaType, httpGrammar_1.httpList, parser_combinators_1.map(mediaTypes => {
    mediaTypes.sort(mediaRangeCompare);
    return mediaTypes.map(mediaTypeToMediaRange);
}), parser_combinators_1.parseWith);
const weightedTokenComparator = ([, a], [, b]) => weightedParamComparator(a, b);
const weightedTokenToToken = ([token]) => token;
const parseWeightedToken = pipe_1.pipe(parser_combinators_1.concat(httpGrammar_1.pToken, httpGrammar_1.pParams), httpGrammar_1.httpList, parser_combinators_1.map(values => {
    values.sort(weightedTokenComparator);
    return values.map(weightedTokenToToken);
}), parser_combinators_1.parseWith);
const parseWeightedTokenHeader = (headers, header) => {
    var _a;
    const rawValue = httpHeaders_1.getHeaderValue(headers, header);
    return option_1.isSome(rawValue) ? (_a = parseWeightedToken(rawValue)) !== null && _a !== void 0 ? _a : [] : [];
};
exports.parseHttpPreferencesFromHeaders = (headers) => {
    var _a;
    const acceptedCharsets = parseWeightedTokenHeader(headers, "Accept-Charset");
    const acceptedEncodings = parseWeightedTokenHeader(headers, "Accept-Encoding");
    const acceptedLanguages = parseWeightedTokenHeader(headers, "Accept-Language");
    const rawAccept = httpHeaders_1.getHeaderValue(headers, "Accept");
    const acceptedMediaRanges = option_1.isSome(rawAccept)
        ? (_a = parseAccept(rawAccept)) !== null && _a !== void 0 ? _a : [] : [];
    const isUndefined = acceptedCharsets.length === 0 &&
        acceptedEncodings.length === 0 &&
        acceptedLanguages.length === 0 &&
        acceptedMediaRanges.length === 0;
    return isUndefined
        ? option_1.none
        : {
            acceptedCharsets,
            acceptedEncodings,
            acceptedLanguages,
            acceptedMediaRanges,
        };
};
const writeWeightedTokenHeader = (header, values, writeHeader) => {
    const length = values.length;
    if (length > 0) {
        const incr = 1000 / length;
        let result = "";
        for (let i = 0; i < length; i++) {
            result += values[i];
            if (i > 0) {
                const q = (i * incr) / 1000;
                result += `; q=${q.toFixed(1)}`;
            }
            if (i < length - 1) {
                result += ", ";
            }
        }
        writeHeader(header, result);
    }
};
exports.writeHttpPreferenceHeaders = (preferences, writeHeader) => {
    const { acceptedCharsets, acceptedEncodings, acceptedLanguages, acceptedMediaRanges, } = preferences;
    writeWeightedTokenHeader("Accept-Charset", acceptedCharsets, writeHeader);
    writeWeightedTokenHeader("Accept-Encoding", acceptedEncodings, writeHeader);
    writeWeightedTokenHeader("Accept-Language", acceptedLanguages, writeHeader);
    const tokenizedMediaRanges = acceptedMediaRanges.map(({ type, subtype }) => `${type}/${subtype}`);
    writeWeightedTokenHeader("Accept", tokenizedMediaRanges, writeHeader);
};
//# sourceMappingURL=httpPreferences.js.map