"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const option_1 = require("@reactive-js/option");
const httpContent_1 = require("./httpContent");
const httpHeaders_1 = require("./httpHeaders");
const httpPreferences_1 = require("./httpPreferences");
const httpRequestPreconditions_1 = require("./httpRequestPreconditions");
exports.createHttpRequest = (method, uri, options = {}) => {
    var _a, _b, _c, _d;
    return (Object.assign(Object.assign({}, options), { expectContinue: (_a = options.expectContinue) !== null && _a !== void 0 ? _a : false, headers: (_b = options.headers) !== null && _b !== void 0 ? _b : {}, httpVersionMajor: (_c = options.httpVersionMajor) !== null && _c !== void 0 ? _c : 1, httpVersionMinor: (_d = options.httpVersionMinor) !== null && _d !== void 0 ? _d : 1, method, uri: typeof uri === "string" ? new URL(uri) : uri }));
};
exports.createRedirectHttpRequest = (response) => request => {
    const { content, method } = request;
    const { location, statusCode } = response;
    const redirectToGet = statusCode === 303 ||
        ((statusCode === 301 ||
            302 === 302) &&
            method === "POST");
    return Object.assign(Object.assign({}, request), { content: redirectToGet ? option_1.none : content, method: redirectToGet ? "GET" : method, uri: location });
};
const parseURIFromHeaders = (protocol, path, httpVersionMajor, headers) => {
    const forwardedProtocol = httpHeaders_1.getHeaderValue(headers, "X-Forwarded-Proto");
    const uriProtocol = option_1.isSome(forwardedProtocol)
        ? forwardedProtocol.split(/\s*,\s*/, 1)[0]
        : protocol;
    const forwardedHost = httpHeaders_1.getHeaderValue(headers, "X-Forwarded-Host");
    const http2Authority = headers[":authority"];
    const http1Host = httpHeaders_1.getHeaderValue(headers, "Host");
    const unfilteredHost = option_1.isSome(forwardedHost)
        ? forwardedHost
        : option_1.isSome(http2Authority) && httpVersionMajor >= 2
            ? http2Authority
            : option_1.isSome(http1Host)
                ? http1Host
                : "";
    const host = unfilteredHost.split(/\s*,\s*/, 1)[0];
    return new URL(`${uriProtocol}://${host}${path !== null && path !== void 0 ? path : ""}`);
};
exports.parseHttpRequestFromHeaders = ({ method, path, headers, httpVersionMajor, httpVersionMinor, isTransportSecure, body, }) => {
    const content = httpContent_1.parseHttpContentFromHeaders(headers, body);
    const rawExpectHeader = httpHeaders_1.getHeaderValue(headers, "Expect");
    const expectContinue = rawExpectHeader === "100-continue";
    const preconditions = httpRequestPreconditions_1.parseHttpRequestPreconditionsFromHeaders(headers);
    const preferences = httpPreferences_1.parseHttpPreferencesFromHeaders(headers);
    const protocol = isTransportSecure ? "https" : "http";
    const uri = parseURIFromHeaders(protocol, path, httpVersionMajor, headers);
    return {
        content,
        expectContinue,
        headers,
        httpVersionMajor,
        httpVersionMinor,
        isTransportSecure,
        method,
        preconditions,
        preferences,
        uri,
    };
};
exports.writeHttpRequestHeaders = ({ content, expectContinue, headers, preconditions, preferences, }, writeHeader) => {
    if (expectContinue) {
        writeHeader("Expect", "100-continue");
    }
    if (option_1.isSome(content)) {
        httpContent_1.writeHttpContentHeaders(content, writeHeader);
    }
    if (option_1.isSome(preconditions)) {
        httpRequestPreconditions_1.writeHttpRequestPreconditionsHeaders(preconditions, writeHeader);
    }
    if (option_1.isSome(preferences)) {
        httpPreferences_1.writeHttpPreferenceHeaders(preferences, writeHeader);
    }
    httpHeaders_1.writeHttpHeaders(headers, writeHeader);
};
exports.disallowProtocolAndHostForwarding = () => request => {
    const { httpVersionMajor, headers: oldHeaders, isTransportSecure, uri: oldUri, } = request;
    const { "x-forwarded-proto": xForwardedProto, "x-forwarded-host": xForwardedHost } = oldHeaders, headers = tslib_1.__rest(oldHeaders, ["x-forwarded-proto", "x-forwarded-host"]);
    const protocol = isTransportSecure ? "https" : "http";
    if (option_1.isNone(xForwardedProto) && option_1.isNone(xForwardedHost)) {
        return request;
    }
    else {
        const path = oldUri.pathname;
        const uri = parseURIFromHeaders(protocol, path, httpVersionMajor, headers);
        return Object.assign(Object.assign({}, request), { uri,
            headers });
    }
};
exports.httpRequestToUntypedHeaders = (request) => {
    const headers = {};
    exports.writeHttpRequestHeaders(request, (header, value) => (headers[header] = value));
    return headers;
};
//# sourceMappingURL=httpRequest.js.map