"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const option_1 = require("@reactive-js/option");
const parser_combinators_1 = require("@reactive-js/parser-combinators");
const pipe_1 = require("@reactive-js/pipe");
const entityTag_1 = require("./entityTag");
const httpDateTime_1 = require("./httpDateTime");
const httpHeaders_1 = require("./httpHeaders");
const httpGrammar_1 = require("./httpGrammar");
const writeEtagPreferenceHeader = (header, value, writeHeader) => {
    if (option_1.isSome(value)) {
        writeHeader(header, Array.isArray(value) ? value.map(entityTag_1.entityTagToString).join(",") : "*");
    }
};
const writeDateHeader = (header, value, writeHeader) => {
    if (option_1.isSome(value)) {
        writeHeader(header, httpDateTime_1.httpDateTimeToString(value));
    }
};
exports.writeHttpRequestPreconditionsHeaders = ({ ifMatch, ifModifiedSince, ifNoneMatch, ifUnmodifiedSince, ifRange, }, writeHeader) => {
    writeEtagPreferenceHeader("If-Match", ifMatch, writeHeader);
    writeEtagPreferenceHeader("If-None-Match", ifNoneMatch, writeHeader);
    writeDateHeader("If-Modified-Since", ifModifiedSince, writeHeader);
    writeDateHeader("If-Unmodified-Since", ifUnmodifiedSince, writeHeader);
    if (option_1.isSome(ifRange)) {
        writeHeader("If-Range", typeof ifRange === "number"
            ? httpDateTime_1.httpDateTimeToString(ifRange)
            : entityTag_1.entityTagToString(ifRange));
    }
};
const parseETagPreference = pipe_1.pipe(entityTag_1.pETag, httpGrammar_1.httpList, parser_combinators_1.or(pipe_1.pipe(parser_combinators_1.pAsterisk, parser_combinators_1.mapTo("*"))), parser_combinators_1.parseWith);
const parseOptionalETagPreference = (headers, header) => {
    const value = httpHeaders_1.getHeaderValue(headers, header);
    return option_1.isSome(value) ? parseETagPreference(value) : option_1.none;
};
const parseOptionalDatePreference = (headers, header) => { var _a; return pipe_1.pipe((_a = httpHeaders_1.getHeaderValue(headers, header)) !== null && _a !== void 0 ? _a : "", httpDateTime_1.parseHttpDateTime); };
exports.parseHttpRequestPreconditionsFromHeaders = (headers) => {
    const ifMatch = parseOptionalETagPreference(headers, "If-Match");
    const ifNoneMatch = parseOptionalETagPreference(headers, "If-None-Match");
    const ifModifiedSince = parseOptionalDatePreference(headers, "If-Modified-Since");
    const ifUnmodifiedSince = parseOptionalDatePreference(headers, "If-Unmodified-Since");
    const ifRangeHeader = httpHeaders_1.getHeaderValue(headers, "If-Range");
    const ifRange = option_1.isSome(ifRangeHeader)
        ?
            httpDateTime_1.parseHttpDateTime(ifRangeHeader) || entityTag_1.parseETag(ifRangeHeader)
        : option_1.none;
    const isUndefined = option_1.isNone(ifMatch) &&
        option_1.isNone(ifNoneMatch) &&
        option_1.isNone(ifModifiedSince) &&
        option_1.isNone(ifUnmodifiedSince) &&
        option_1.isNone(ifRangeHeader);
    return isUndefined
        ? option_1.none
        : {
            ifMatch,
            ifModifiedSince,
            ifNoneMatch,
            ifUnmodifiedSince,
            ifRange,
        };
};
//# sourceMappingURL=httpRequestPreconditions.js.map