"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fresh_1 = tslib_1.__importDefault(require("fresh"));
const option_1 = require("@reactive-js/option");
const httpContent_1 = require("./httpContent");
const httpDateTime_1 = require("./httpDateTime");
const entityTag_1 = require("./entityTag");
const httpHeaders_1 = require("./httpHeaders");
const httpPreferences_1 = require("./httpPreferences");
exports.createHttpResponse = (statusCode, options = {}) => {
    var _a, _b;
    return (Object.assign(Object.assign({}, options), { headers: (_a = options.headers) !== null && _a !== void 0 ? _a : {}, statusCode, vary: (_b = options.vary) !== null && _b !== void 0 ? _b : [] }));
};
exports.parseHttpResponseFromHeaders = (statusCode, headers, body) => {
    var _a, _b, _c;
    const content = httpContent_1.parseHttpContentFromHeaders(headers, body);
    const etag = entityTag_1.parseETag((_a = httpHeaders_1.getHeaderValue(headers, "ETag")) !== null && _a !== void 0 ? _a : "");
    const expires = httpDateTime_1.parseHttpDateTime((_b = httpHeaders_1.getHeaderValue(headers, "Expires")) !== null && _b !== void 0 ? _b : "");
    const lastModified = httpDateTime_1.parseHttpDateTime((_c = httpHeaders_1.getHeaderValue(headers, "Last-Modified")) !== null && _c !== void 0 ? _c : "");
    const locationHeader = headers.location;
    const location = option_1.isSome(locationHeader) ? new URL(locationHeader) : option_1.none;
    const preferences = httpPreferences_1.parseHttpPreferencesFromHeaders(headers);
    const vary = [];
    return {
        etag,
        expires,
        lastModified,
        content,
        headers,
        location,
        preferences,
        statusCode,
        vary,
    };
};
const writeCoreHttpResponseHeaders = ({ etag, expires, headers, lastModified, location, preferences, vary, }, writeHeader) => {
    if (option_1.isSome(etag)) {
        writeHeader("ETag", entityTag_1.entityTagToString(etag));
    }
    if (option_1.isSome(expires)) {
        writeHeader("Expires", httpDateTime_1.httpDateTimeToString(expires));
    }
    if (option_1.isSome(lastModified)) {
        writeHeader("Last-Modified", httpDateTime_1.httpDateTimeToString(lastModified));
    }
    if (option_1.isSome(location)) {
        writeHeader("Location", location.toString());
    }
    if (option_1.isSome(preferences)) {
        httpPreferences_1.writeHttpPreferenceHeaders(preferences, writeHeader);
    }
    if (vary.length > 0) {
        writeHeader("Vary", vary.join(","));
    }
    httpHeaders_1.writeHttpHeaders(headers, writeHeader);
};
exports.writeHttpResponseHeaders = (response, writeHeader) => {
    const { content } = response;
    writeCoreHttpResponseHeaders(response, writeHeader);
    if (option_1.isSome(content)) {
        httpContent_1.writeHttpContentHeaders(content, writeHeader);
    }
};
exports.checkIfNotModified = ({ headers: reqHeaders, method, }) => response => {
    const { statusCode, content: _ } = response, requestWithoutContent = tslib_1.__rest(response, ["statusCode", "content"]);
    const methodSupportsFresh = method === "GET" || method === "HEAD";
    const statusCodeSupportsFresh = statusCode >= 200 && statusCode < 300;
    const headers = {};
    writeCoreHttpResponseHeaders(response, (k, v) => {
        headers[k.toLowerCase()] = v;
    });
    return methodSupportsFresh &&
        statusCodeSupportsFresh &&
        fresh_1.default(reqHeaders, headers)
        ? exports.createHttpResponse(304, requestWithoutContent)
        : response;
};
//# sourceMappingURL=httpResponse.js.map