export interface URILike {
    readonly hash: string;
    readonly host: string;
    readonly hostname: string;
    readonly href: string;
    readonly origin: string;
    readonly pathname: string;
    readonly port: string;
    readonly protocol: string;
    readonly search: string;
    toString(): string;
}
export declare const enum HttpMethod {
    GET = "GET",
    HEAD = "HEAD",
    POST = "POST",
    PUT = "PUT",
    DELETE = "DELETE"
}
export declare const enum HttpStatusCode {
    Continue = 100,
    SwitchingProtocols = 101,
    Processing = 102,
    OK = 200,
    Created = 201,
    Accepted = 202,
    NonAuthoritativeInformation = 203,
    NoContent = 204,
    ResetContent = 205,
    PartialContent = 206,
    MultiStatus = 207,
    AlreadyReported = 208,
    IMUsed = 226,
    MultipleChoices = 300,
    MovedPermanently = 301,
    Found = 302,
    SeeOther = 303,
    NotModified = 304,
    UseProxy = 305,
    TemporaryRedirect = 307,
    PermanentRedirect = 308,
    BadRequest = 400,
    Unauthorized = 401,
    Forbidden = 403,
    NotFound = 404,
    MethodNotAllowed = 405,
    NotAcceptable = 406,
    ProxyAuthenticationRequired = 407,
    RequestTimeout = 408,
    Conflict = 409,
    Gone = 410,
    LengthRequired = 411,
    PreconditionFailed = 412,
    RequestEntityTooLarge = 413,
    RequestURITooLong = 414,
    UnsupportedMediaType = 415,
    RequestedRangeNotSatisfiable = 416,
    ExpectationFailed = 417,
    UnprocessableEntity = 422,
    Locked = 423,
    FailedDependency = 424,
    UpgradeRequired = 426,
    PreconditionRequired = 428,
    TooManyRequests = 429,
    RequestHeaderFieldsTooLarge = 431,
    UnavailableForLegalReasons = 451,
    InternalServerError = 500,
    NotImplemented = 501,
    BadGateway = 502,
    ServiceUnavailable = 503,
    GatewayTimeout = 504,
    HTTPVersionNotSupported = 505,
    VariantAlsoNegotiates = 506,
    InsufficientStorage = 507,
    LoopDetected = 508,
    NotExtended = 510,
    NetworkAuthenticationRequired = 511
}
export declare const enum HttpContentEncoding {
    Brotli = "br",
    Compress = "compress",
    Deflate = "deflate",
    GZip = "gzip",
    Identity = "identity"
}
export declare type HttpHeaders = {
    readonly [header: string]: string;
};
export declare type MediaType = {
    readonly type: string;
    readonly subtype: string;
    readonly params: {
        readonly [key: string]: string;
    };
};
export declare type MediaRange = {
    readonly type: string | "*";
    readonly subtype: string | "*";
};
export declare type HttpContent<T> = {
    readonly body: T;
    readonly contentEncodings: readonly HttpContentEncoding[];
    readonly contentLength: number;
    readonly contentType: MediaType;
};
export declare type HttpPreferences = {
    readonly acceptedCharsets: readonly string[];
    readonly acceptedEncodings: readonly HttpContentEncoding[];
    readonly acceptedLanguages: readonly string[];
    readonly acceptedMediaRanges: readonly MediaRange[];
};
export declare type EntityTag = {
    readonly isWeak: boolean;
    readonly tag: string;
};
export declare type HttpDateTime = number;
export declare type HttpRequestPreconditions = {
    readonly ifMatch?: readonly EntityTag[] | "*";
    readonly ifModifiedSince?: HttpDateTime;
    readonly ifNoneMatch?: readonly EntityTag[] | "*";
    readonly ifUnmodifiedSince?: HttpDateTime;
    readonly ifRange?: EntityTag | HttpDateTime;
};
export declare type HttpRequest<T> = {
    readonly content?: T;
    readonly expectContinue: boolean;
    readonly headers: HttpHeaders;
    readonly method: HttpMethod;
    readonly preconditions?: HttpRequestPreconditions;
    readonly preferences?: HttpPreferences;
    readonly uri: URILike;
    readonly httpVersionMajor: number;
    readonly httpVersionMinor: number;
};
export declare type HttpContentRequest<T> = HttpRequest<HttpContent<T>>;
export declare type HttpServerRequest<T> = HttpContentRequest<T> & {
    readonly isTransportSecure: boolean;
};
export declare type HttpResponse<T> = {
    readonly content?: T;
    readonly etag?: EntityTag;
    readonly expires?: HttpDateTime;
    readonly headers: HttpHeaders;
    readonly lastModified?: HttpDateTime;
    readonly location?: URILike;
    readonly preferences?: HttpPreferences;
    readonly statusCode: HttpStatusCode;
    readonly vary: readonly string[];
};
export declare type HttpContentResponse<T> = HttpResponse<HttpContent<T>>;
