import { isNone, none } from "@reactive-js/option";
import { parseWith } from "@reactive-js/parser-combinators";
import { pipe } from "@reactive-js/pipe";
import { pToken, httpList } from "./httpGrammar";
import { getHeaderValue } from "./httpHeaders";
import { parseMediaType, mediaTypeToString } from "./mediaType";
const parseTokenList = pipe(pToken, httpList, parseWith);
export const parseHttpContentFromHeaders = (headers, body) => {
    var _a, _b, _c;
    const contentEncodingString = (_a = getHeaderValue(headers, "Content-Encoding")) !== null && _a !== void 0 ? _a : "";
    const contentEncodings = parseTokenList(contentEncodingString);
    const contentLengthHeader = (_b = getHeaderValue(headers, "Content-Length")) !== null && _b !== void 0 ? _b : "-1";
    const contentLength = ~~contentLengthHeader;
    const contentType = parseMediaType((_c = getHeaderValue(headers, "Content-Type")) !== null && _c !== void 0 ? _c : "");
    return isNone(contentType)
        ? none
        : {
            body,
            contentEncodings,
            contentLength,
            contentType,
        };
};
export const writeHttpContentHeaders = (content, writeHeader) => {
    const { contentLength, contentType, contentEncodings } = content;
    if (contentLength > 0) {
        writeHeader("Content-Length", contentLength.toString(10));
    }
    writeHeader("Content-Type", mediaTypeToString(contentType));
    if (contentEncodings.length > 0) {
        writeHeader("Content-Encoding", contentEncodings.join(", "));
    }
};
//# sourceMappingURL=httpContent.js.map