import { __rest } from "tslib";
import { isNone, isSome, none } from "@reactive-js/option";
import { writeHttpContentHeaders, parseHttpContentFromHeaders, } from "./httpContent";
import { writeHttpHeaders, getHeaderValue, } from "./httpHeaders";
import { parseHttpPreferencesFromHeaders, writeHttpPreferenceHeaders, } from "./httpPreferences";
import { writeHttpRequestPreconditionsHeaders, parseHttpRequestPreconditionsFromHeaders, } from "./httpRequestPreconditions";
export const createHttpRequest = (method, uri, options = {}) => {
    var _a, _b, _c, _d;
    return (Object.assign(Object.assign({}, options), { expectContinue: (_a = options.expectContinue) !== null && _a !== void 0 ? _a : false, headers: (_b = options.headers) !== null && _b !== void 0 ? _b : {}, httpVersionMajor: (_c = options.httpVersionMajor) !== null && _c !== void 0 ? _c : 1, httpVersionMinor: (_d = options.httpVersionMinor) !== null && _d !== void 0 ? _d : 1, method, uri: typeof uri === "string" ? new URL(uri) : uri }));
};
export const createRedirectHttpRequest = (response) => request => {
    const { content, method } = request;
    const { location, statusCode } = response;
    const redirectToGet = statusCode === 303 ||
        ((statusCode === 301 ||
            302 === 302) &&
            method === "POST");
    return Object.assign(Object.assign({}, request), { content: redirectToGet ? none : content, method: redirectToGet ? "GET" : method, uri: location });
};
const parseURIFromHeaders = (protocol, path, httpVersionMajor, headers) => {
    const forwardedProtocol = getHeaderValue(headers, "X-Forwarded-Proto");
    const uriProtocol = isSome(forwardedProtocol)
        ? forwardedProtocol.split(/\s*,\s*/, 1)[0]
        : protocol;
    const forwardedHost = getHeaderValue(headers, "X-Forwarded-Host");
    const http2Authority = headers[":authority"];
    const http1Host = getHeaderValue(headers, "Host");
    const unfilteredHost = isSome(forwardedHost)
        ? forwardedHost
        : isSome(http2Authority) && httpVersionMajor >= 2
            ? http2Authority
            : isSome(http1Host)
                ? http1Host
                : "";
    const host = unfilteredHost.split(/\s*,\s*/, 1)[0];
    return new URL(`${uriProtocol}://${host}${path !== null && path !== void 0 ? path : ""}`);
};
export const parseHttpRequestFromHeaders = ({ method, path, headers, httpVersionMajor, httpVersionMinor, isTransportSecure, body, }) => {
    const content = parseHttpContentFromHeaders(headers, body);
    const rawExpectHeader = getHeaderValue(headers, "Expect");
    const expectContinue = rawExpectHeader === "100-continue";
    const preconditions = parseHttpRequestPreconditionsFromHeaders(headers);
    const preferences = parseHttpPreferencesFromHeaders(headers);
    const protocol = isTransportSecure ? "https" : "http";
    const uri = parseURIFromHeaders(protocol, path, httpVersionMajor, headers);
    return {
        content,
        expectContinue,
        headers,
        httpVersionMajor,
        httpVersionMinor,
        isTransportSecure,
        method,
        preconditions,
        preferences,
        uri,
    };
};
export const writeHttpRequestHeaders = ({ content, expectContinue, headers, preconditions, preferences, }, writeHeader) => {
    if (expectContinue) {
        writeHeader("Expect", "100-continue");
    }
    if (isSome(content)) {
        writeHttpContentHeaders(content, writeHeader);
    }
    if (isSome(preconditions)) {
        writeHttpRequestPreconditionsHeaders(preconditions, writeHeader);
    }
    if (isSome(preferences)) {
        writeHttpPreferenceHeaders(preferences, writeHeader);
    }
    writeHttpHeaders(headers, writeHeader);
};
export const disallowProtocolAndHostForwarding = () => request => {
    const { httpVersionMajor, headers: oldHeaders, isTransportSecure, uri: oldUri, } = request;
    const { "x-forwarded-proto": xForwardedProto, "x-forwarded-host": xForwardedHost } = oldHeaders, headers = __rest(oldHeaders, ["x-forwarded-proto", "x-forwarded-host"]);
    const protocol = isTransportSecure ? "https" : "http";
    if (isNone(xForwardedProto) && isNone(xForwardedHost)) {
        return request;
    }
    else {
        const path = oldUri.pathname;
        const uri = parseURIFromHeaders(protocol, path, httpVersionMajor, headers);
        return Object.assign(Object.assign({}, request), { uri,
            headers });
    }
};
export const httpRequestToUntypedHeaders = (request) => {
    const headers = {};
    writeHttpRequestHeaders(request, (header, value) => (headers[header] = value));
    return headers;
};
//# sourceMappingURL=httpRequest.js.map