import { isNone, isSome, none } from "@reactive-js/option";
import { parseWith, pAsterisk, or, mapTo, } from "@reactive-js/parser-combinators";
import { pipe } from "@reactive-js/pipe";
import { entityTagToString, pETag, parseETag } from "./entityTag";
import { httpDateTimeToString, parseHttpDateTime } from "./httpDateTime";
import { getHeaderValue } from "./httpHeaders";
import { httpList } from "./httpGrammar";
const writeEtagPreferenceHeader = (header, value, writeHeader) => {
    if (isSome(value)) {
        writeHeader(header, Array.isArray(value) ? value.map(entityTagToString).join(",") : "*");
    }
};
const writeDateHeader = (header, value, writeHeader) => {
    if (isSome(value)) {
        writeHeader(header, httpDateTimeToString(value));
    }
};
export const writeHttpRequestPreconditionsHeaders = ({ ifMatch, ifModifiedSince, ifNoneMatch, ifUnmodifiedSince, ifRange, }, writeHeader) => {
    writeEtagPreferenceHeader("If-Match", ifMatch, writeHeader);
    writeEtagPreferenceHeader("If-None-Match", ifNoneMatch, writeHeader);
    writeDateHeader("If-Modified-Since", ifModifiedSince, writeHeader);
    writeDateHeader("If-Unmodified-Since", ifUnmodifiedSince, writeHeader);
    if (isSome(ifRange)) {
        writeHeader("If-Range", typeof ifRange === "number"
            ? httpDateTimeToString(ifRange)
            : entityTagToString(ifRange));
    }
};
const parseETagPreference = pipe(pETag, httpList, or(pipe(pAsterisk, mapTo("*"))), parseWith);
const parseOptionalETagPreference = (headers, header) => {
    const value = getHeaderValue(headers, header);
    return isSome(value) ? parseETagPreference(value) : none;
};
const parseOptionalDatePreference = (headers, header) => { var _a; return pipe((_a = getHeaderValue(headers, header)) !== null && _a !== void 0 ? _a : "", parseHttpDateTime); };
export const parseHttpRequestPreconditionsFromHeaders = (headers) => {
    const ifMatch = parseOptionalETagPreference(headers, "If-Match");
    const ifNoneMatch = parseOptionalETagPreference(headers, "If-None-Match");
    const ifModifiedSince = parseOptionalDatePreference(headers, "If-Modified-Since");
    const ifUnmodifiedSince = parseOptionalDatePreference(headers, "If-Unmodified-Since");
    const ifRangeHeader = getHeaderValue(headers, "If-Range");
    const ifRange = isSome(ifRangeHeader)
        ?
            parseHttpDateTime(ifRangeHeader) || parseETag(ifRangeHeader)
        : none;
    const isUndefined = isNone(ifMatch) &&
        isNone(ifNoneMatch) &&
        isNone(ifModifiedSince) &&
        isNone(ifUnmodifiedSince) &&
        isNone(ifRangeHeader);
    return isUndefined
        ? none
        : {
            ifMatch,
            ifModifiedSince,
            ifNoneMatch,
            ifUnmodifiedSince,
            ifRange,
        };
};
//# sourceMappingURL=httpRequestPreconditions.js.map