import { __rest } from "tslib";
import fresh from "fresh";
import { isSome, none } from "@reactive-js/option";
import { writeHttpContentHeaders, parseHttpContentFromHeaders, } from "./httpContent";
import { parseHttpDateTime, httpDateTimeToString } from "./httpDateTime";
import { entityTagToString, parseETag } from "./entityTag";
import { writeHttpHeaders, getHeaderValue, } from "./httpHeaders";
import { parseHttpPreferencesFromHeaders, writeHttpPreferenceHeaders, } from "./httpPreferences";
export const createHttpResponse = (statusCode, options = {}) => {
    var _a, _b;
    return (Object.assign(Object.assign({}, options), { headers: (_a = options.headers) !== null && _a !== void 0 ? _a : {}, statusCode, vary: (_b = options.vary) !== null && _b !== void 0 ? _b : [] }));
};
export const parseHttpResponseFromHeaders = (statusCode, headers, body) => {
    var _a, _b, _c;
    const content = parseHttpContentFromHeaders(headers, body);
    const etag = parseETag((_a = getHeaderValue(headers, "ETag")) !== null && _a !== void 0 ? _a : "");
    const expires = parseHttpDateTime((_b = getHeaderValue(headers, "Expires")) !== null && _b !== void 0 ? _b : "");
    const lastModified = parseHttpDateTime((_c = getHeaderValue(headers, "Last-Modified")) !== null && _c !== void 0 ? _c : "");
    const locationHeader = headers.location;
    const location = isSome(locationHeader) ? new URL(locationHeader) : none;
    const preferences = parseHttpPreferencesFromHeaders(headers);
    const vary = [];
    return {
        etag,
        expires,
        lastModified,
        content,
        headers,
        location,
        preferences,
        statusCode,
        vary,
    };
};
const writeCoreHttpResponseHeaders = ({ etag, expires, headers, lastModified, location, preferences, vary, }, writeHeader) => {
    if (isSome(etag)) {
        writeHeader("ETag", entityTagToString(etag));
    }
    if (isSome(expires)) {
        writeHeader("Expires", httpDateTimeToString(expires));
    }
    if (isSome(lastModified)) {
        writeHeader("Last-Modified", httpDateTimeToString(lastModified));
    }
    if (isSome(location)) {
        writeHeader("Location", location.toString());
    }
    if (isSome(preferences)) {
        writeHttpPreferenceHeaders(preferences, writeHeader);
    }
    if (vary.length > 0) {
        writeHeader("Vary", vary.join(","));
    }
    writeHttpHeaders(headers, writeHeader);
};
export const writeHttpResponseHeaders = (response, writeHeader) => {
    const { content } = response;
    writeCoreHttpResponseHeaders(response, writeHeader);
    if (isSome(content)) {
        writeHttpContentHeaders(content, writeHeader);
    }
};
export const checkIfNotModified = ({ headers: reqHeaders, method, }) => response => {
    const { statusCode, content: _ } = response, requestWithoutContent = __rest(response, ["statusCode", "content"]);
    const methodSupportsFresh = method === "GET" || method === "HEAD";
    const statusCodeSupportsFresh = statusCode >= 200 && statusCode < 300;
    const headers = {};
    writeCoreHttpResponseHeaders(response, (k, v) => {
        headers[k.toLowerCase()] = v;
    });
    return methodSupportsFresh &&
        statusCodeSupportsFresh &&
        fresh(reqHeaders, headers)
        ? createHttpResponse(304, requestWithoutContent)
        : response;
};
//# sourceMappingURL=httpResponse.js.map